#include <avr/io.h>
#include <avr/boot.h>
#include <avr/pgmspace.h>
#include "lcd.h"

void BOOTLOADER_SECTION writeFlash(uint16_t pageAddress, uint16_t data[], uint16_t size);
void lcdDefineChar_P(const uint8_t * charDefinition, uint8_t code);
void lcdString_P(const char * str);

const char WeekDay1[] PROGMEM = "  ";
const char WeekDay2[] PROGMEM = "pn";
const char WeekDay3[] PROGMEM = "wt";
const char WeekDay4[] PROGMEM = "sr";
const char WeekDay5[] PROGMEM = "cz";
const char WeekDay6[] PROGMEM = "pt";
const char WeekDay7[] PROGMEM = "so";
const char WeekDay8[] PROGMEM = "nd";

PGM_P const WeekDays[] PROGMEM = {
	WeekDay1,
	WeekDay2,
	WeekDay3,
	WeekDay4,
	WeekDay5,
	WeekDay6,
	WeekDay7,
	WeekDay8,
};

#define FLASH_PAGE_SIZE_BYTES 128

const uint8_t customCharBell[8] PROGMEM = {
	0b00000,
	0b00100,
	0b01110,
	0b01110,
	0b01110,
	0b11111,
	0b00100,
	0b00000
};

int main(void) {
	lcdInit();
	lcdInitPrintf();
	//test zapisu danych w stronie nr 100
	uint8_t testData[] = {1, 2, 3, 4, 5};
	writeFlash(FLASH_PAGE_SIZE_BYTES * 100, (uint16_t*) testData, (sizeof(testData) + 1) / 2);
	for (uint8_t i = 0; i < sizeof(testData); i++) {
		uint8_t byte = pgm_read_byte(FLASH_PAGE_SIZE_BYTES * 100 + i);
		printf("%d ", byte);
	}
	//test odczytu definicji znaku zapisanej w staej
	lcdDefineChar_P(customCharBell, 0);
	lcdGotoXY(10, 0);
	lcdWriteData(0);
	//test odczytu cigu znakw z tablicy cigw
	lcdGotoXY(0, 1);
	lcdString_P(pgm_read_ptr(&(WeekDays[2])));
	//test odczytu literau
	lcdString(" ");
	lcdString_P(PSTR("EdW"));
	while(1){ }
}

void lcdDefineChar_P(const uint8_t * charDefinition, uint8_t code) {
	lcdWriteCommand(LCD_COMMAND_SET_CGRAM_ADDRESS + (code % 8) * 8);
	for (uint8_t i = 0; i < 8; i++) {
		lcdWriteData(pgm_read_byte(charDefinition + i));
	}
}

void lcdString_P(const char * str) {
	uint8_t c = 0;
	while ((c = pgm_read_byte(str++)) != 0) {
		lcdWriteData(c);
	}
}

void writeFlash(uint16_t pageAddress, uint16_t data[], uint16_t size) {
	boot_page_erase_safe(pageAddress);
	for (uint16_t i = 0; i < size; i++) {
		boot_page_fill_safe(pageAddress + i *2, data[i]);
	}
	boot_page_write_safe(pageAddress);
	boot_rww_enable_safe();
}
